#!/usr/bin/perl -w
use strict;
use warnings;
use lib '/Users/your/path/to/BioPerl-1.6.1/';

use Bio::SeqIO;
use Bio::SearchIO;
use File::Basename;

#Before using, please edit these variables to appropriate values:
# - $input_dir (directory containing CDS files)
# - $output_dir (directory containing raw BLAST results)
# - $output_dir (directory to contain new, filtered BLAST results)
# - regular expression after "foreach my $file(@files)". Make sure to match correct directory + extension.
# - 	the line is 
# -			$file =~ m/BlastResults\/(\S+).txt/;
# -	As it is here, this searches for the string after 'BlastResults', before the extension '.txt'.
# Directory containing CDS fasta files
my $input_dir = "/Users/your/path/to/ORGANISM/FILTERING/INPUT";

# Individual CDS fasta files
my @seed_seqs = <$input_dir/*.fasta>;

#make giant lookup for gene lengths
my %gene_length;

#Genome sequence files
foreach my $file (@seed_seqs){
    my $seqIO = Bio::SeqIO->new(-file=>$file, -format => 'fasta') || die "Couldn't open $file for reading: $!";
    while(my $seq = $seqIO->next_seq()){
	$gene_length{$seq->id}=length($seq->seq);
    }
}

# Filtering parameters
my $coverage_cutoff = 0.8;
my $evalue_cutoff   = 1e-10;
my $length_cutoff 	= 0.75;

my $count=0;

# directory containing raw BLAST results
my $output_dir = "/Users/your/path/to/ORGANISM/FILTERING/OUTPUT";


#BLAST result files
my @files = <$output_dir/OUTPUT_FILENAME_Blast.txt>;

# Filtered files (which this script generates)
my $output_2 = "/Users/your/path/to/ORGANISM/FILTERING/FILTEROUT";

foreach my $file(@files){
# 	$file =~ m/raw_BLAST_results\/(\S+).fasta/;
# 	my $core = $file;
 #	$file =~ m/BlastResults\/(\S+).txt/;
 #	my $core = $1;
	open(OUT,">$output_2/FILE_ALL_BLAST_filtered.txt") or die("Couldn't open $file for reading: $!");
	#print "$output_2/$core.txt\n";
	#die
	
	#open(OUT,">$output_2/$file") 
	#	print STDERR "Printint to >$output_2/AllResults_filtered.txt\n";

#    print STDERR "processing :" . $file."\n";
    #exit;
    my $searchio = Bio::SearchIO->new(
	-format => 'blasttable',
	-file   => "$file"
	);
    while ( my $result = $searchio->next_result() ) {
	my $query_id = $result->query_name();
	
	my $query_len = get_gene_length($query_id);
	next unless defined($query_len);
	foreach my $hit(@{$result->{_hits}}){
	    $hit->{-query_len}=$query_len;
	}
	#process the Bio::Search::Hit::GenericHit
	while ( my $hit = $result->next_hit ) {
	    my $hit_id         = $hit->name();
	    next if $hit_id eq $query_id;
	    my $hit_len = get_gene_length($hit_id);
	    next unless defined($hit_len);
	    $hit->length($hit_len);
	    
	    my $evalue         = $hit->significance();
	    my $identity     = $hit->frac_identical();
	    my $query_coverage = $hit->frac_aligned_query();
	    my $hit_coverage   = $hit->frac_aligned_hit();

		if (   ( $query_coverage > $coverage_cutoff )
		   && ( $hit_coverage > $coverage_cutoff )
		   && ( $evalue < $evalue_cutoff )
		   && ( $hit_len > ($length_cutoff * $query_len) )
		   && ( $query_len > ($length_cutoff * $hit_len) ) )
	    {

		#write post-processed information to output file 'OUT'
		print OUT "$query_id\t$hit_id\t$identity\t$evalue\t$query_coverage\t$hit_coverage\n";
	    }
	}
    }
}

sub get_gene_length{
    my $id = shift;
    unless(exists($gene_length{$id})){
	warn "Could not find gene: $id!\n";
	return;
    }
    return $gene_length{$id};
}
